/*
====================================================================

Program:             APMMONIT.C 

Description:         Advanced Power Management monitor

Date created:        00/07/27

Date last updated:   00/07/27

Notes:               This program will display battery information every second.
                     It is used mainly to verify the working state of the battery
                     and whether the battery suffers from any sudden power loss.


(C) Copyright 2000 Circuit Design Corporation
====================================================================
*/



#define INCL_DOSDEVIOCTL
#include <cdc.h>



int main(void)
{

 APIRET rc;

 HFILE apm_handle = 0;

 ULONG action_taken;

 USHORT apm_rc;

 ULONG parm_length,
       data_length;

 #pragma pack(1)

 struct {

    USHORT ParmLength;
    USHORT PowerFlags;
    BYTE   ACStatus;
    BYTE   BatteryStatus;
    BYTE   BatteryLife;

 } power_status;

 #pragma pack()


   setbuf(stdout, 0);

   printf("\nAPM Battery Monitor\n");
   printf("(C) 2000 Copyright Circuit Design Corporation\n\n");

   printf("Accessing APM driver...\n");
  
   rc = DosOpen("APM$",
                &apm_handle,
                &action_taken,
                0,
                FILE_NORMAL,
                OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
                OPEN_FLAGS_FAIL_ON_ERROR | OPEN_SHARE_DENYNONE | OPEN_ACCESS_READWRITE,
                0);
  
   if (rc != NO_ERROR) {
      printf("APM driver not found!  Error code = %ld  Action taken = %ld", rc, action_taken);
      return ERROR;
   }

   printf("Beginning monitoring..\n\n");

   while (TRUE) {

      memset(&power_status, 0, sizeof(power_status));
      power_status.ParmLength = sizeof(power_status);

      parm_length = 
      data_length = 0;

      rc = DosDevIOCtl(apm_handle,
                       IOCTL_POWER,
                       POWER_GETPOWERSTATUS,
                       &power_status,
                       sizeof(power_status),
                       &parm_length,
                       &apm_rc,
                       sizeof(apm_rc),
                       &data_length);

      if (rc != NO_ERROR) {
         printf("APM driver access failed!  Error code = %ld\n", rc);
         return ERROR;
      }

      if (rc != NO_ERROR) {
         printf("APM driver failed!  Error code = %ld\n", apm_rc);
         return ERROR;
      }

      if (power_status.PowerFlags == 0) {
         printf("APM driver is currently DISABLED!\n");
         return ERROR;
      }

      printf("Power source: ");

      switch (power_status.ACStatus) {

         case 0:
            printf("Battery   ");
            break;

         case 1:
            printf("AC        ");
            break;

         default:
            printf("Unknown   ");
            break;

      }

      printf("Battery status: ");

      switch (power_status.BatteryStatus) {

         case 0:
            printf("High     ");
            break;

         case 1:
            printf("Low      ");
            break;

         case 2:
            printf("Critical ");
            break;

         case 3:
            printf("Charging ");
            break;

         default:
            printf("Unknown  ");
            break;

      }

      switch (power_status.BatteryLife) {

         case 255:
            printf("Unknown %%\n");
            break;

         default:
            printf("%d%%\n", power_status.BatteryLife);
            break;

      }

      DosSleep(1000);

   }


   return OK;

}

